# frozen_string_literal: true

module BaselineRedRpm
  module Instruments
    module RackModule
      def call(env)
        req = ::Rack::Request.new(env)

        #if ::BaselineRedRpm::Tracer.in_trace? &&
        #  ::BaselineRedRpm.config.instrumentation[:rack][:trace_middleware]
        #  BaselineRedRpm::Tracer.trace("rack-middleware") do |span|
        #    span.set_tag "type", "web"
        #    span.set_tag "domain", req.host
        #    span.set_tag "url", req.path
        #    span.set_tag "class", @app.class.name
        #
        #    status, headers, body = @app.call env
        #  end
        #else
        span = BaselineRedRpm.tracer.start_span("rack")
        span.set_tag "type", "web"
        span.set_tag "domain", req.host
        span.set_tag "url", req.path
        span.set_tag "class", @app.class.name
        span.set_tag "backtrace", ::BaselineRedRpm::Backtrace.backtrace
        span.set_tag "source", ::BaselineRedRpm::Backtrace.source_extract

        @app.call(env)
      rescue Exception => e
        if span
          span.set_tag('error', true)
          span.log_error(e)
        end
        raise
      ensure
        span.finish if span
      end
    end
  end
end

if ::BaselineRedRpm.config.instrumentation[:rack][:enabled]
  ::BaselineRedRpm.logger.info "Initializing rack tracer."

  if ::BaselineRedRpm.config.instrumentation[:rack][:trace_middleware]
    ::BaselineRedRpm.logger.info "Initializing rack-middleware tracer."
  end

  module BaselineRedRpm
    module Instruments
      class Rack
        attr_reader :app

        #include BaselineRedRpm::Instruments::RackModule

        def initialize(app)
          @app = app
        end

        def call(env)
          req = ::Rack::Request.new(env)

          unless ignore_path?(req.path)
            extracted_ctx = BaselineRedRpm.tracer.extract(OpenTracing::FORMAT_RACK, env)
            BaselineRedRpm::Tracer.sample!(extracted_ctx, !!req.params["app-perf-sample"])

            if BaselineRedRpm::Tracer.tracing?
              span = BaselineRedRpm.tracer.start_span(@app.class.name, :child_of => extracted_ctx, tags: {
                "component" => "Rack",
                "span.kind" => "client"
              })
              BaselineRedRpm::Utils.log_source_and_backtrace(span, :rack)
            end
          end

          status, headers, response = @app.call(env)

          if span
            span.set_tag "peer.address", req.host
            span.set_tag "peer.port", req.port
            span.set_tag "http.method", req.request_method
            span.set_tag "http.url", req.path
            span.set_tag "http.status_code", status
          end

          [status, headers, response]
        rescue Exception => e
          if span
            span.set_tag('error', true)
            span.log_error(e)
          end
          raise
        ensure
          span.finish if span
          BaselineRedRpm::Tracer.sample_off!
        end

        def ignore_path?(path)
          path.to_s =~ BaselineRedRpm.config.ignore_paths
        end
      end
    end
  end
=begin
  module ActionDispatch
    class MiddlewareStack
      class BaselineRedRack
        include BaselineRedRpm::Instruments::RackModule

        def initialize(app)
          @app = app
        end
      end

      class Middleware
        def build(app)
          BaselineRedRack.new(klass.new(app, *args, &block))
        end
      end

      def build(app = nil, &block)
        app ||= block
        raise "BaselineRedRack#build requires an app" unless app
        middlewares.reverse.inject(BaselineRedRack.new(app)) {|a, e| e.build(a)}
      end
    end
  end
=end
end
